#!/usr/bin/env node

const fs = require('fs');
const process = require('process');
const sourceMap = require('source-map');

async function main() {
  if (process.argv.length < 4) {
    console.error('Usage: ' + `${process.argv[1]} <sourcemap> <output file>`);
    process.exit(1);
  }

  const fileName = process.argv[2];
  const destination = process.argv[3];
  let consumer = undefined;
  try {
    const mapping = fs.readFileSync(fileName, 'utf-8');
    consumer = await new sourceMap.SourceMapConsumer(mapping);
  } catch(err) {
    console.error(err);
    process.exit(1);
  }

  let simplified = {
    mappings: [],
    files: [],
    version: 1,
  };

  let files = {};

  consumer.eachMapping(function (m) {
    let fileId = undefined;

    if (m.source in files) {
      fileId = files[m.source];
    } else {
      fileId = Object.keys(files).length;
      files[m.source] = fileId;
    }

    // map (line, column) -> (file, line, column, name)
    let mappingData = [
      m.generatedLine,
      m.generatedColumn,
      fileId,
      m.originalLine,
      m.originalColumn,
      m.name,
    ]

    simplified.mappings.push(mappingData);
  }, undefined, sourceMap.SourceMapConsumer.GENERATED_ORDER);

  consumer.destroy();

  simplified.files = Object.keys(files).sort(function(l, r) {
    return files[l] - files[r];
  });

  try {
    fs.writeFileSync(`${destination}.temp`, JSON.stringify(simplified), 'utf-8');
    fs.renameSync(`${destination}.temp`, destination);
  } catch(err) {
    console.error(err);
    process.exit(2);
  }
}

main();
